class Cursor:
    def __init__(self, cur):
        self.cur = cur

    def __aiter__(self):
        return self

    async def __anext__(self):
        try:
            return self.cur.next()
        except StopIteration:
            raise StopAsyncIteration

    async def to_list(self, length=None):
        res = []
        ind = 0
        for i in self.cur:
            if length is not None and length == ind:
                break
            res.append(i)
            ind += 1
        return res



from mongita.collection import (
    Collection as MongitaCollection,
    _validate_filter,
    _get_item_from_doc,
)
from mongita.common import support_alert
from mongita.errors import MongitaError

from beanita.cursor import Cursor
from beanita.results import UpdateResult


class Collection(MongitaCollection):
    UNIMPLEMENTED = [
        "aggregate",
        "aggregate_raw_batches",
        "bulk_write",
        "codec_options",
        "ensure_index",
        "estimated_document_count",
        "find_one_and_delete",
        "find_one_and_replace",
        "find_one_and_update",
        "find_raw_batches",
        "inline_map_reduce",
        "list_indexes",
        "map_reduce",
        "next",
        "options",
        "read_concern",
        "read_preference",
        "rename",
        "watch",
    ]

    @support_alert
    async def index_information(self):
        ret = {"_id_": {"key": [("_id", 1)]}}
        metadata = self.__get_metadata()
        for idx in metadata.get("indexes", {}).values():
            ret[idx["_id"]] = {"key": [(idx["key_str"], idx["direction"])]}
        return ret

    async def create_indexes(self, indexes):
        return []  # TODO create indexes here

    @support_alert
    async def insert_one(self, document, session):
        return super().insert_one(document)

    @support_alert
    async def insert_many(self, documents, ordered=True, session=None):
        return super().insert_many(documents, ordered)

    @support_alert
    async def find_one(
        self, filter=None, sort=None, skip=None, session=None, projection=None
    ):
        return super().find_one(filter, sort, skip)

    @support_alert
    def find(
        self,
        filter=None,
        sort=None,
        limit=None,
        skip=None,
        projection=None,
        session=None,
    ):
        print(filter)
        res = super().find(filter, sort, limit, skip)
        print(res)
        return Cursor(res)

    @support_alert
    async def update_one(self, filter, update, upsert=False, session=None):
        return super().update_one(filter, update, upsert)

    @support_alert
    async def update_many(self, filter, update, upsert=False, session=None):
        return super().update_many(filter, update, upsert)

    @support_alert
    async def replace_one(
        self, filter, replacement, upsert=False, session=None
    ):
        res = super().replace_one(filter, replacement, upsert)
        return UpdateResult.from_mongita_result(res)

    @support_alert
    async def delete_one(self, filter, session=None):
        return super().delete_one(filter)

    @support_alert
    async def delete_many(self, filter, session=None):
        return super().delete_many(filter)

    @support_alert
    async def count_documents(self, filter):
        return super().count_documents(filter)

    @support_alert
    async def distinct(self, key, filter=None, session=None):
        """
        Given a key, return all distinct documents matching the key

        :param key str:
        :param filter dict|None:
        :rtype: list[str]
        """

        if not isinstance(key, str):
            raise MongitaError("The 'key' parameter must be a string")
        filter = filter or {}
        _validate_filter(filter)
        uniq = set()
        for doc in await self.find(filter).to_list():
            uniq.add(_get_item_from_doc(doc, key))
        uniq.discard(None)
        return list(uniq)

    async def drop(self):
        await self.delete_many({})

    async def drop_indexes(self):
        pass  # TODO add index dropping
        # for index_name in (await self.index_information()).keys():
        #     self.drop_index(index_name)



class InsertOneResult:
    def __init__(self, inserted_id):
        self.acknowledged = True
        self.inserted_id = inserted_id


class InsertManyResult:
    def __init__(self, documents):
        self.acknowledged = True
        self.inserted_ids = [d["_id"] for d in documents]


class UpdateResult:
    def __init__(
        self, matched_count, modified_count, upserted_id=None, **kwargs
    ):
        self.acknowledged = True
        self.matched_count = matched_count
        self.modified_count = modified_count
        self.upserted_id = upserted_id
        self.updatedExisting = modified_count

    @property
    def raw_result(self):
        return self.__dict__

    @classmethod
    def from_mongita_result(cls, res):
        return cls(**res.__dict__)


class DeleteResult:
    def __init__(self, deleted_count):
        self.acknowledged = True
        self.deleted_count = deleted_count



from mongita import MongitaClientDisk
from mongita.common import ok_name
from mongita.errors import InvalidName

from beanita.db import Database


class Client(MongitaClientDisk):
    def __getitem__(self, db_name):
        try:
            return self._cache[db_name]
        except KeyError:
            if not ok_name(db_name):
                raise InvalidName("Database cannot be named %r." % db_name)
            db = Database(db_name, self)
            self._cache[db_name] = db
            return db



from mongita.common import ok_name
from mongita.database import Database as MongitaDatabase
from mongita.errors import InvalidName

from beanita.collection import Collection


class Database(MongitaDatabase):
    UNIMPLEMENTED = [
        "aggregate",
        "codec_options",
        "create_collection",
        "dereference",
        "get_collection",
        "next",
        "profiling_info",
        "profiling_level",
        "read_concern",
        "read_preference",
        "set_profiling_level",
        "validate_collection",
        "watch",
        "with_options",
        "write_concern",
    ]

    def __getitem__(self, collection_name):
        try:
            return self._cache[collection_name]
        except KeyError:
            if not ok_name(collection_name):
                raise InvalidName(
                    "Collection cannot be named %r." % collection_name
                )
            coll = Collection(collection_name, self)
            self._cache[collection_name] = coll
            return coll

    async def command(self, *args, **kwargs):
        return {"version": "4.4"}

    async def list_collection_names(self):
        return super().list_collection_names()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['beanita']

package_data = \
{'': ['*']}

install_requires = \
['mongita>=1.1.1,<2.0.0']

setup_kwargs = {
    'name': 'beanita',
    'version': '0.1.0',
    'description': 'Local MongoDB-like database, based on Mongita and prepared to work with Beanie ODM',
    'long_description': '## Beanita\n\nLocal MongoDB-like database, based on [Mongita](https://github.com/scottrogowski/mongita) and prepared to work with [Beanie ODM](https://github.com/roman-right/beanie)\n\nI highly recommend using it only for experiment purposes. It is safer to use a real MongoDB database and for testing, and for production.\n\n### Init\n\n```python\nfrom beanie import init_beanie, Document\nfrom beanita import Client\n\n\nclass Sample(Document):\n    name: str\n\n\nasync def init_database():\n    cli = Client("LOCAL_DIRECTORY")\n    db = cli["DATABASE_NAME"]\n    await init_beanie(\n        database=db,\n        document_models=[Sample],\n    )\n```\n\n### Not supported\n\n- Links\n- Aggregations\n- Union Documents\n- other features, that were not implemented in Mongita',
    'author': 'Roman Right',
    'author_email': 'roman-right@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/roman-right/beanita',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
